@@ -40,8 +40,10 @@ from typing import (
     Union,
 )
 
-from primer3 import thermoanalysis  # type: ignore
-from primer3 import argdefaults
+from primer3 import (  # type: ignore
+    argdefaults,
+    thermoanalysis,
+)
 
 DEFAULT_P3_ARGS = argdefaults.Primer3PyArguments()
 THERMO_ANALYSIS = thermoanalysis.ThermoAnalysis()
@@ -61,7 +63,7 @@ def calc_hairpin(
         max_loop: int = DEFAULT_P3_ARGS.max_loop,
         output_structure: bool = False,
 ):
-    ''' Calculate the hairpin formation thermodynamics of a DNA sequence.
+    '''Calculate the hairpin formation thermodynamics of a DNA sequence.
 
     **Note that the maximum length of ``seq`` is 60 bp.** This is a cap
     suggested by the Primer3 team as the longest reasonable sequence length for
@@ -143,7 +145,7 @@ def calc_homodimer(
         max_loop: int = DEFAULT_P3_ARGS.max_loop,
         output_structure: bool = False,
 ):
-    ''' Calculate the homodimerization thermodynamics of a DNA sequence.
+    '''Calculate the homodimerization thermodynamics of a DNA sequence.
 
     **Note that the maximum length of ``seq`` is 60 bp.** This is a cap imposed
     by Primer3 as the longest reasonable sequence length for which
@@ -317,7 +319,7 @@ def calc_end_stability(
         temp_c: Union[float, int] = DEFAULT_P3_ARGS.temp_c,
         max_loop: int = DEFAULT_P3_ARGS.max_loop,
 ) -> thermoanalysis.ThermoResult:
-    ''' Calculate the 3' end stability of DNA sequence `seq1` against DNA
+    '''Calculate the 3' end stability of DNA sequence `seq1` against DNA
     sequence `seq2`.
 
     **Note that at least one of the two sequences must by <60 bp in length.**
@@ -542,6 +544,7 @@ def design_primers(
     Returns:
         A dictionary of Primer3 results (should be identical to the expected
         BoulderIO output from ``primer3_main``)
+
     '''
     return THERMO_ANALYSIS.run_design(
         global_args=global_args,
