@@ -37,12 +37,11 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <limits.h>
 #include <math.h>
+#include <stdio.h>
 #include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
 #include "oligotm.h"
 
-#define T_KELVIN 273.15;
+#define T_KELVIN 273.15
 #define A_CHAR 'A'
 #define G_CHAR 'G'
 #define T_CHAR 'T'
@@ -98,7 +97,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 /* Table 1 (old parameters):
- * See table 2 in the paper [Breslauer KJ, Frank R, Blcker H and
+ * See table 2 in the paper [Breslauer KJ, Frank R, Bloecker H and
  * Marky LA (1986) "Predicting DNA duplex stability from the base
  * sequence" Proc Natl Acad Sci 83:4746-50
  * http://dx.doi.org/10.1073/pnas.83.11.3746]
@@ -301,38 +300,64 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 /* Calculate the melting temperature of oligo s.  See
    oligotm.h for documentation of arguments.
 */
-double
+tm_ret
 oligotm(const  char *s,
-     double DNA_nM,
-     double K_mM,
-     double divalent_conc,
-     double dntp_conc,
-     tm_method_type  tm_method,
-     salt_correction_type salt_corrections)
-{
+    double DNA_nM,
+    double K_mM,
+    double divalent_conc,
+    double dntp_conc,
+    double dmso_conc,
+    double dmso_fact,
+    double formamide_conc,
+    tm_method_type  tm_method,
+    salt_correction_type salt_corrections,
+    double annealing_temp
+) {
   register int dh = 0, ds = 0;
   register char c;
   double delta_H, delta_S;
-  double Tm; /* Melting temperature */
+  double ddG, ka;
+  tm_ret ret;
+  ret.bound = OLIGOTM_ERROR;
   double correction;
-  int len, sym;
-  const char* seq = s;
-  if(divalent_to_monovalent(divalent_conc, dntp_conc) == OLIGOTM_ERROR) {
-    return OLIGOTM_ERROR;
-  }
+  int len, sym, i;
+  int GC_count = 0;
+  const char* d = s;
 
+  /* NOTE: primer3-py block for intentionally commented out for debugging
+  * `oligotm_main` as the upstream argument parser has issues/bugs with argument
+  * name collisions
+  */
+  // printf(
+  //   "%f, %f, %f, %f, %f, %f, %f, %f\n",
+  //   DNA_nM, K_mM, divalent_conc, dntp_conc,
+  //   dmso_conc, dmso_fact, formamide_conc, annealing_temp
+  // );
+
+  if(divalent_to_monovalent(divalent_conc, dntp_conc) == OLIGOTM_ERROR) {
+     ret.Tm = OLIGOTM_ERROR;
+     return ret;
+  }
   /** K_mM = K_mM + divalent_to_monovalent(divalent_conc, dntp_conc); **/
   if (tm_method != breslauer_auto
       && tm_method != santalucia_auto) {
-    return OLIGOTM_ERROR;
+     ret.Tm = OLIGOTM_ERROR;
+     return ret;
   }
   if (salt_corrections != schildkraut
       && salt_corrections != santalucia
       && salt_corrections != owczarzy) {
-    return OLIGOTM_ERROR;
+     ret.Tm = OLIGOTM_ERROR;
+     return ret;
+  }
+  len = (strlen(s)-1);
+  if(formamide_conc != 0.0) {
+    for (i = 0; i < len + 1; i++) {
+      if ((strncmp("G", s + i, 1) == 0) || (strncmp("C", s + i, 1) == 0)) {
+        GC_count++;
+      }
+    }
   }
-
-  len = ((int)strlen(s)-1);
 
   sym = symmetry(s); /*Add symmetry correction if seq is symmetrical*/
   if( tm_method == breslauer_auto ) {
@@ -345,26 +370,26 @@ oligotm(const  char *s,
 
     /** Terminal AT penalty **/
 
-    if (    (strncmp("A", s, 1) == 0)
-        ||  (strncmp("T", s, 1) == 0) )  {
+    if(strncmp("A", s, 1)==0
+       || strncmp("T", s, 1)==0)  {
       ds += -41;
       dh += -23;
-    } else if (   (strncmp("C", s, 1) == 0)
-              ||  (strncmp("G", s, 1) == 0) ) {
+    } else if (strncmp("C", s, 1)==0
+               || strncmp("G", s, 1)==0) {
       ds += 28;
       dh += -1;
     }
-    s += len;
-    if (  (strncmp("T", s, 1) == 0)
-       || (strncmp("A", s, 1) == 0) ) {
+    s+=len;
+    if(strncmp("T", s, 1)==0
+       || strncmp("A", s, 1)==0) {
       ds += -41;
       dh += -23;
-    } else if ( (strncmp("C", s, 1) == 0)
-              ||(strncmp("G", s, 1) == 0) ) {
+    } else if (strncmp("C", s, 1)==0
+               || strncmp("G", s, 1)==0) {
       ds += 28;
       dh += -1;
     }
-    s -= len;
+    s-=len;
   }
   /* Use a finite-state machine (DFA) to calucluate dh and ds for s. */
   c = *s; s++;
@@ -395,121 +420,142 @@ oligotm(const  char *s,
   }
 
 
- DONE:  /* dh and ds are now computed for the given sequence. */
+DONE:  /* dh and ds are now computed for the given sequence. */
   delta_H = dh * -100.0;  /*
-         * Nearest-neighbor thermodynamic values for dh
-         * are given in 100 cal/mol of interaction.
-         */
+                           * Nearest-neighbor thermodynamic values for dh
+                           * are given in 100 cal/mol of interaction.
+                           */
   delta_S = ds * -0.1;     /*
-          * Nearest-neighbor thermodynamic values for ds
-          * are in in .1 cal/K per mol of interaction.
-          */
-  Tm=0;  /* Melting temperature */
+                            * Nearest-neighbor thermodynamic values for ds
+                            * are in in .1 cal/K per mol of interaction.
+                            */
+  ret.Tm=0;  /* Melting temperature */
   len=len+1;
 
    /**********************************************/
   if (salt_corrections == schildkraut) {
      K_mM = K_mM + divalent_to_monovalent(divalent_conc, dntp_conc);
-     correction = 16.6 * log10(K_mM/1000.0) - T_KELVIN;
-     Tm = delta_H / (delta_S + 1.987 * log(DNA_nM/4000000000.0)) + correction;
-  } else if (salt_corrections== santalucia) {
-    K_mM = K_mM + divalent_to_monovalent(divalent_conc, dntp_conc);
-    delta_S = delta_S + 0.368 * (len - 1) * log(K_mM / 1000.0 );
-    if(sym == 1) { /* primer is symmetrical */
-      /* Equation A */
-      Tm = delta_H / (delta_S + 1.987 * log(DNA_nM/1000000000.0)) - T_KELVIN;
-    } else {
-      /* Equation B */
-      Tm = delta_H / (delta_S + 1.987 * log(DNA_nM/4000000000.0)) - T_KELVIN;
-    }
+     correction = 16.6 * log10(K_mM/1000.0);
+     ret.Tm = delta_H / (delta_S + 1.987 * log(DNA_nM/4000000000.0)) + correction - T_KELVIN;
+     ret.Tm -= dmso_conc * dmso_fact;
+     ret.Tm += (0.453 * ((double) GC_count) / len - 2.88) * formamide_conc;
+     if (annealing_temp > 0.0) {
+        ddG = delta_H - (annealing_temp - correction + T_KELVIN) * delta_S;
+        ka = exp(-ddG / (1.987 * (annealing_temp - correction + T_KELVIN)));
+        ret.bound = (1 / (1 + sqrt(1/((DNA_nM/4000000000.0) * ka)))) * 100;
+     }
+  } else if (salt_corrections == santalucia) {
+     K_mM = K_mM + divalent_to_monovalent(divalent_conc, dntp_conc);
+     delta_S = delta_S + 0.368 * (len - 1) * log(K_mM / 1000.0 );
+     if(sym == 1) { /* primer is symmetrical */
+        /* Equation A */
+        ret.Tm = delta_H / (delta_S + 1.987 * log(DNA_nM/1000000000.0)) - T_KELVIN;
+        ret.Tm -= dmso_conc * dmso_fact;
+        ret.Tm += (0.453 * ((double) GC_count) / len - 2.88) * formamide_conc;
+        if (annealing_temp > 0.0) {
+           ddG = delta_H - (annealing_temp + T_KELVIN) * delta_S;
+           ka = exp(-ddG / (1.987 * (annealing_temp + T_KELVIN)));
+           ret.bound = (1 / (1 + sqrt(1/((DNA_nM/1000000000.0) * ka)))) * 100;
+        }
+     } else {
+        /* Equation B */
+        ret.Tm = delta_H / (delta_S + 1.987 * log(DNA_nM/4000000000.0)) - T_KELVIN;
+        ret.Tm -= dmso_conc * dmso_fact;
+        ret.Tm += (0.453 * ((double) GC_count) / len - 2.88) * formamide_conc;
+        if (annealing_temp > 0.0) {
+           ddG = delta_H - (annealing_temp + T_KELVIN) * delta_S;
+           ka = exp(-ddG / (1.987 * (annealing_temp + T_KELVIN)));
+           ret.bound = (1 / (1 + sqrt(1/((DNA_nM/4000000000.0) * ka)))) * 100;
+        }
+     }
   } else if (salt_corrections == owczarzy) {
-    double gcPercent=0;
-    double free_divalent; /* conc of divalent cations minus dNTP conc */
-    int i;
+     double gcPercent=0;
+     double free_divalent; /* conc of divalent cations minus dNTP conc */
+     int i;
+     for(i = 0; i <= len && d != NULL && *d != '\0';) {
+        if(*d == 'C' || *d == 'G') {
+           gcPercent++;
+        }
+        d++;
+        i++;
+     }
+     gcPercent = (double)gcPercent/((double)len);
+     /**** BEGIN: UPDATED SALT BY OWCZARZY *****/
+      /* different salt corrections for monovalent (Owczarzy et al.,2004)
+      and divalent cations (Owczarzy et al.,2008)
+      */
+     /* competition bw magnesium and monovalent cations, see Owczarzy et al., 2008 Figure 9 and Equation 16 */
 
-    /**** BEGIN: UPDATED SALT BY OWCZARZY *****/
-    /* different salt corrections for monovalent (Owczarzy et al.,2004)
-      and divalent cations (Owczarzy et al.,2008) */
-    /* competition bw magnesium and monovalent cations, see Owczarzy et al., 2008 Figure 9 and Equation 16 */
-
-    static const double crossover_point = 0.22; /* depending on the value of div_monov_ratio respect
-               to value of crossover_point Eq 16 (divalent corr, Owczarzy et al., 2008)
-               or Eq 22 (monovalent corr, Owczarzy et al., 2004) should be used */
-    double div_monov_ratio;
-    static double a = 0, b = 0, c = 0, d = 0, e = 0, f = 0, g = 0;
-
-    for(i = 0; i <= len && seq != NULL && seq != '\0';) {
-      if(*seq == 'C' || *seq == 'G') {
-        gcPercent++;
+     static const double crossover_point = 0.22; /* depending on the value of div_monov_ratio respect
+                                             to value of crossover_point Eq 16 (divalent corr, Owczarzy et al., 2008)
+                                             or Eq 22 (monovalent corr, Owczarzy et al., 2004) should be used */
+     double div_monov_ratio;
+     if(dntp_conc >= divalent_conc) {
+        free_divalent = 0.00000000001; /* to not to get log(0) */
+     } else {
+        free_divalent = (divalent_conc - dntp_conc)/1000.0;
+     }
+     static double a = 0,b = 0,c = 0,d = 0,e = 0,f = 0,g = 0;
+      if(K_mM==0) {
+         div_monov_ratio = 6.0;
+      } else {
+         div_monov_ratio = (sqrt(free_divalent))/(K_mM/1000); /* if conc of monov cations is provided
+                                                                    a ratio is calculated to further calculate
+                                                                    the _correct_ correction */
       }
-      seq++;
-      i++;
-    }
-    gcPercent = (double)gcPercent/((double)len);
+     if (div_monov_ratio < crossover_point) {
+        /* use only monovalent salt correction, Eq 22 (Owczarzy et al., 2004) */
+        correction
+          = (((4.29 * gcPercent) - 3.95) * pow(10,-5) * log(K_mM / 1000.0))
+            + (9.40 * pow(10,-6) * (pow(log(K_mM / 1000.0),2)));
+     } else {
+        /* magnesium effects are dominant, Eq 16 (Owczarzy et al., 2008) is used */
+        b =- 9.11 * pow(10,-6);
+        c = 6.26 * pow(10,-5);
+        e =- 4.82 * pow(10,-4);
+        f = 5.25 * pow(10,-4);
+        a = 3.92 * pow(10,-5);
+        d = 1.42 * pow(10,-5);
+        g = 8.31 * pow(10,-5);
+        if(div_monov_ratio < 6.0) {
+           /* in particular ratio of conc of monov and div cations
+            *             some parameters of Eq 16 must be corrected (a,d,g) */
+           a = 3.92 * pow(10,-5) * (0.843 - (0.352 * sqrt(K_mM/1000.0) * log(K_mM/1000.0)));
+           d = 1.42 * pow(10,-5) * (1.279 - 4.03 * pow(10,-3) * log(K_mM/1000.0) - 8.03 * pow(10,-3) * pow(log(K_mM/1000.0),2));
+           g = 8.31 * pow(10,-5) * (0.486 - 0.258 * log(K_mM/1000.0) + 5.25 * pow(10,-3) * pow(log(K_mM/1000.0),3));
+        }
 
-    if(dntp_conc >= divalent_conc) {
-      free_divalent = 0.00000000001; /* to not to get log(0) */
-    } else {
-      free_divalent = (divalent_conc - dntp_conc)/1000.0;
-    }
-
-    if(K_mM==0) {
-      div_monov_ratio = 6.0;
-    } else {
-      div_monov_ratio = (sqrt(free_divalent))/(K_mM/1000); /* if conc of monov cations is provided
-                    a ratio is calculated to further calculate
-                    the _correct_ correction */
-    }
-    if (div_monov_ratio < crossover_point) {
-      /* use only monovalent salt correction, Eq 22 (Owczarzy et al., 2004) */
-      correction
-        = (((4.29 * gcPercent) - 3.95) * pow(10,-5) * log(K_mM / 1000.0))
-          + (9.40 * pow(10,-6) * (pow(log(K_mM / 1000.0),2)));
-    } else {
-      /* magnesium effects are dominant, Eq 16 (Owczarzy et al., 2008) is used */
-      b =- 9.11 * pow(10,-6);
-      c = 6.26 * pow(10,-5);
-      e =- 4.82 * pow(10,-4);
-      f = 5.25 * pow(10,-4);
-      a = 3.92 * pow(10,-5);
-      d = 1.42 * pow(10,-5);
-      g = 8.31 * pow(10,-5);
-      if(div_monov_ratio < 6.0) {
-        /* in particular ratio of conc of monov and div cations
-        *             some parameters of Eq 16 must be corrected (a,d,g) */
-        a = 3.92 * pow(10,-5) * (0.843 - (0.352 * sqrt(K_mM/1000.0) * log(K_mM/1000.0)));
-        d = 1.42 * pow(10,-5) * (1.279 - 4.03 * pow(10,-3) * log(K_mM/1000.0) - 8.03 * pow(10,-3) * pow(log(K_mM/1000.0),2));
-        g = 8.31 * pow(10,-5) * (0.486 - 0.258 * log(K_mM/1000.0) + 5.25 * pow(10,-3) * pow(log(K_mM/1000.0),3));
-      }
-
-      correction = a + (b * log(free_divalent))
-                  + gcPercent * (c + (d * log(free_divalent)))
-                  + (1/(2 * (len - 1))) * (e + (f * log(free_divalent))
-                  + g * (pow((log(free_divalent)),2)));
-    }
-    /**** END: UPDATED SALT BY OWCZARZY *****/
-    if (sym == 1) {
-      /* primer is symmetrical */
-      /* Equation A */
-      Tm = 1/((1/(delta_H
-                    /
-              (delta_S + 1.9872 * log(DNA_nM/1000000000.0)))) + correction) - T_KELVIN;
-    } else {
-      /* Equation B */
-      Tm = 1/((1/(delta_H
-          /
-        (delta_S + 1.9872 * log(DNA_nM/4000000000.0)))) + correction) - T_KELVIN;
-    }
+        correction = a + (b * log(free_divalent))
+                       + gcPercent * (c + (d * log(free_divalent)))
+                       + (1/(2 * (len - 1))) * (e + (f * log(free_divalent))
+                       + g * (pow((log(free_divalent)),2)));
+     }
+     /**** END: UPDATED SALT BY OWCZARZY *****/
+     if (sym == 1) {
+           /* primer is symmetrical */
+        /* Equation A */
+        ret.Tm = 1/((1/(delta_H
+                        /
+                        (delta_S + 1.9872 * log(DNA_nM/1000000000.0)))) + correction) - T_KELVIN;
+     } else {
+        /* Equation B */
+        ret.Tm = 1/((1/(delta_H
+                        /
+                        (delta_S + 1.9872 * log(DNA_nM/4000000000.0)))) + correction) - T_KELVIN;
+     }
+     ret.Tm -= dmso_conc * dmso_fact;
+     ret.Tm += (0.453 * ((double) GC_count) / len - 2.88) * formamide_conc;
   } /* END else if (salt_corrections == owczarzy) { */
 
 
-   /***************************************/
-   return Tm;
+  /***************************************/
+  return ret;
 ERROR:  /*
-    * length of s was less than 2 or there was an illegal character in
-    * s.
-    */
-  return OLIGOTM_ERROR;
+          * length of s was less than 2 or there was an illegal character in
+          * s.
+          */
+  ret.Tm = OLIGOTM_ERROR;
+  return ret;
 }
 #undef DO_PAIR
 #undef DO_PAIR2
@@ -530,33 +576,32 @@ double
 oligodg(const char *s,      /* The sequence. */
     int tm_method)
 {
-  register int dg = 0;
-  register char c;
+   register int dg = 0;
+   register char c;
 
-  if (    (tm_method != breslauer_auto)
-      &&  (tm_method != santalucia_auto)) {
+  if (tm_method != breslauer_auto
+      && tm_method != santalucia_auto)
     return OLIGOTM_ERROR;
-  }
 
-  /* Use a finite-state machine (DFA) to calucluate dg s. */
-  c = *s; s++;
-  if(tm_method != breslauer_auto) {
-    dg = -1960; /* Initial dG */
-    if(c == 'A' || c == 'T')  {
-      dg += -50; /* terminal AT penalty */
-    }
-    if (c == 'A') goto A_STATE2;
-    else if (c == 'G') goto G_STATE2;
-    else if (c == 'T') goto T_STATE2;
-    else if (c == 'C') goto C_STATE2;
-    else if (c == 'N') goto N_STATE2;
-    else goto ERROR;
-    STATE2(A);
-    STATE2(T);
-    STATE2(G);
-    STATE2(C);
-    STATE2(N);
-  } else {
+   /* Use a finite-state machine (DFA) to calucluate dg s. */
+   c = *s; s++;
+   if(tm_method != breslauer_auto) {
+      dg=-1960; /* Initial dG */
+      if(c == 'A' || c == 'T')  {
+         dg += -50; /* terminal AT penalty */
+      }
+      if (c == 'A') goto A_STATE2;
+      else if (c == 'G') goto G_STATE2;
+      else if (c == 'T') goto T_STATE2;
+      else if (c == 'C') goto C_STATE2;
+      else if (c == 'N') goto N_STATE2;
+      else goto ERROR;
+      STATE2(A);
+      STATE2(T);
+      STATE2(G);
+      STATE2(C);
+      STATE2(N);
+     } else {
     if (c == 'A') goto A_STATE;
     else if (c == 'G') goto G_STATE;
     else if (c == 'T') goto T_STATE;
@@ -569,38 +614,37 @@ oligodg(const char *s,      /* The sequence. */
     STATE(C);
     STATE(N);
 
-  }
+     }
 DONE:  /* dg is now computed for the given sequence. */
-  if(tm_method != breslauer_auto) {
-    int sym;
-    --s; --s; c = *s;
-    if(c == 'A' || c == 'T')  {
-      dg += -50; /* terminal AT penalty */
-    }
-    sym = symmetry(s);
-    if(sym==1)   {
-      dg +=-430; /* symmetry correction for dG */
-    }
-  }
-  return dg / 1000.0;
+   if(tm_method != breslauer_auto) {
+      int sym;
+      --s; --s; c = *s;
+      if(c == 'A' || c == 'T')  {
+         dg += -50; /* terminal AT penalty */
+      }
+      sym = symmetry(s);
+      if(sym==1)   {
+         dg +=-430; /* symmetry correction for dG */
+      }
+   }
+   return dg / 1000.0;
 
-ERROR:  /*
-        * length of s was less than 2 or there was an illegal character in
-        * s.
-        */
-  return OLIGOTM_ERROR;
+ ERROR:  /*
+          * length of s was less than 2 or there was an illegal character in
+          * s.
+          */
+    return OLIGOTM_ERROR;
 }
 
 double end_oligodg(const char *s,
   int len, /* The number of characters to return. */
   int tm_method)
 {
-  int x = (int)strlen(s);
+  int x = strlen(s);
 
-  if (    (tm_method != breslauer_auto)
-      &&  (tm_method != santalucia_auto)) {
+  if (tm_method != breslauer_auto
+      && tm_method != santalucia_auto)
     return OLIGOTM_ERROR;
-}
 
   return
     x < len
@@ -609,141 +653,133 @@ double end_oligodg(const char *s,
 }
 
 /* See oligotm.h for documentation of arguments. */
-double seqtm(const  char *seq,
-  double dna_conc,
-  double salt_conc,
-  double divalent_conc,
-  double dntp_conc,
-  int    nn_max_len,
-  tm_method_type tm_method,
-  salt_correction_type salt_corrections)
+tm_ret seqtm(const  char *seq,
+             double dna_conc,
+             double salt_conc,
+             double divalent_conc,
+             double dntp_conc,
+             double dmso_conc,
+             double dmso_fact,
+             double formamide_conc,
+             int    nn_max_len,
+             tm_method_type tm_method,
+             salt_correction_type salt_corrections,
+             double annealing_temp)
 {
-  int i = 0;
-  double result = OLIGOTM_ERROR;
-  int len = (int) strlen(seq);
-  char* seqcopy = NULL;
-
-  if (tm_method != breslauer_auto
-      && tm_method != santalucia_auto) {
-    return OLIGOTM_ERROR;
-  }
-
-  if (salt_corrections != schildkraut
-      && salt_corrections != santalucia
-      && salt_corrections != owczarzy) {
-        return OLIGOTM_ERROR;
-  }
-  /* copy string to ensure upper case for character handling*/
-  seqcopy = (char*) malloc((len + 1) * sizeof(char));
-  if (seqcopy == NULL) {
-    return result;
-  }
-  strcpy(seqcopy, seq);
-  for(i = 0; i < len; i++) { seqcopy[i] = toupper(seqcopy[i]); }
-
-  if (len > nn_max_len) {
-    result = long_seq_tm(seqcopy,
-                          0,
-                          len,
-                          salt_conc,
-                          divalent_conc,
-                          dntp_conc);
-  } else {
-    result = oligotm(seqcopy,
-                    dna_conc,
-                    salt_conc,
-                    divalent_conc,
-                    dntp_conc,
-                    tm_method,
-                    salt_corrections);
-  }
-  free(seqcopy);
-  seqcopy = NULL;
-  return result;
+  int len = strlen(seq);
+  tm_ret ret;
+  ret.bound = OLIGOTM_ERROR;
+  ret.Tm = OLIGOTM_ERROR;
+   if (
+      (tm_method != breslauer_auto)
+      && (tm_method != santalucia_auto)
+   ) {
+      return ret;
+   }
+   if (
+         (salt_corrections != schildkraut)
+         && (salt_corrections != santalucia)
+         && (salt_corrections != owczarzy)
+   ) {
+      return ret;
+   }
+   if (len > nn_max_len) {
+      return long_seq_tm(seq, 0, len, salt_conc, divalent_conc, dntp_conc,
+                       dmso_conc, dmso_fact, formamide_conc);
+   } else {
+      // printf("nn_max_len is %d\n", nn_max_len);
+      return oligotm(seq, dna_conc, salt_conc, divalent_conc, dntp_conc, dmso_conc,
+                   dmso_fact, formamide_conc, tm_method, salt_corrections, annealing_temp);
+   }
 }
 
 /* See oligotm.h for documentation on this function and the formula it
    uses. */
-double
+tm_ret
 long_seq_tm(
   const char *s,
   int start,
   int len,
   double salt_conc,
   double divalent_conc,
-  double dntp_conc)
-{
+  double dntp_conc,
+  double dmso_conc,
+  double dmso_fact,
+  double formamide_conc
+) {
   int GC_count = 0;
   const char *p, *end;
+  tm_ret ret;
+  ret.bound = OLIGOTM_ERROR;
+  ret.Tm = OLIGOTM_ERROR;
 
   if (divalent_to_monovalent(divalent_conc, dntp_conc) == OLIGOTM_ERROR)
-    return OLIGOTM_ERROR;
+    return ret;
 
   salt_conc = salt_conc + divalent_to_monovalent(divalent_conc, dntp_conc);
 
   if ((unsigned) (start + len) > strlen(s) || start < 0 || len <= 0)
-    return OLIGOTM_ERROR;
+    return ret;
   end = &s[start + len];
   /* Length <= 0 is nonsensical. */
   for (p = &s[start]; p < end; p++) {
-    if ('G' == *p || 'C' == *p) {
+    if ('G' == *p || 'C' == *p)
       GC_count++;
-    }
   }
 
-  return
-    81.5
-    + (16.6 * log10(salt_conc / 1000.0))
-    + (41.0 * (((double) GC_count) / len))
-    - (600.0 / len);
+  ret.Tm = 81.5 - dmso_conc * dmso_fact
+                + (0.453 * ((double) GC_count) / len - 2.88) * formamide_conc
+                + (16.6 * log10(salt_conc / 1000.0))
+                + (41.0 * (((double) GC_count) / len))
+                - (600.0 / len);
+
+  return ret;
 }
 
  /* Return 1 if string is symmetrical, 0 otherwise. */
 int symmetry(const char* seq) {
-  register char s;
-  register char e;
-  const char *seq_end=seq;
-  int i = 0;
-  int seq_len = (int)strlen(seq);
-  int mp = seq_len / 2;
-  if((seq_len % 2) == 1) {
-    return 0;
-  }
-  seq_end += seq_len;
-  seq_end--;
-  while(i < mp) {
-    i++;
-    s = *seq;
-    e = *seq_end;
-    if (    (s=='A' && e!='T')
-        ||  (s=='T' && e!='A')
-        ||  (e=='A' && s!='T')
-        ||  (e=='T' && s!='A')) {
+   register char s;
+   register char e;
+   const char *seq_end=seq;
+   int i = 0;
+   int seq_len=strlen(seq);
+   int mp = seq_len/2;
+   if(seq_len%2==1) {
       return 0;
-    }
-    if (  (s=='C' && e!='G')
-      ||  (s=='G' && e!='C')
-      ||  (e=='C' && s!='G')
-      ||  (e=='G' && s!='C')) {
-      return 0;
-    }
-    seq++;
-    seq_end--;
-  }
-  return 1;
+   }
+   seq_end+=seq_len;
+   seq_end--;
+   while(i<mp) {
+      i++;
+      s=*seq;
+      e=*seq_end;
+      if ((s=='A' && e!='T')
+          || (s=='T' && e!='A')
+          || (e=='A' && s!='T')
+          || (e=='T' && s!='A')) {
+         return 0;
+      }
+      if ((s=='C' && e!='G')
+          || (s=='G' && e!='C')
+          || (e=='C' && s!='G')
+          || (e=='G' && s!='C')) {
+         return 0;
+      }
+      seq++;
+      seq_end--;
+   }
+   return 1;
 }
 
 /* Convert divalent salt concentration to monovalent */
 double divalent_to_monovalent(double divalent,
-            double dntp)
+                              double dntp)
 {
-    if (divalent==0) { dntp=0; }
-    if (divalent < 0 || dntp < 0) { return OLIGOTM_ERROR; }
-    if (divalent < dntp) {
-      /* According to theory, melting temperature does not depend on
-      * divalent cations
-      */
-      divalent = dntp;
-    }
-    return 120*(sqrt(divalent-dntp));
+   if(divalent==0) dntp=0;
+   if(divalent<0 || dntp<0) return OLIGOTM_ERROR;
+   if(divalent<dntp)
+     /* According to theory, melting temperature does not depend on
+        divalent cations */
+     divalent=dntp;
+   return 120 * (sqrt(divalent-dntp));
 }
