@@ -1,6 +1,6 @@
-/* Copyright (c) 1996,1997,1998,1999,2000,2001,2004,2006,2007,2008,2009,2010
-                 2011,2012
- Whitehead Institute for Biomedical Research, Steve Rozen
+/* Copyright (c) 1996 - 2018
+ Whitehead Institute for Biomedical Research, Steve Rozen, Andreas Untergasser
+
  (http://purl.com/STEVEROZEN/), and Helen Skaletsky
  All rights reserved.
 
@@ -90,8 +90,8 @@ typedef enum thal_alignment_type {
 } thal_alignment_type;
 
 /* Structure for passing arguments to THermodynamic ALignment calculation */
-typedef struct {
-   int debug; /* if non zero, print debugging info to stderr */
+typedef struct thal_args {
+   // int debug; /* if non zero, print debugging info to stderr */ update NC
    thal_alignment_type type; /* one of the
 	      1 THAL_ANY, (by default)
 	      2 THAL_END1,
@@ -103,13 +103,13 @@ typedef struct {
    double dntp; /* concentration of dNTP-s */
    double dna_conc; /* concentration of oligonucleotides */
    double temp; /* temperature from which hairpin structures will be calculated */
-   int temponly; /* if non zero, print only temperature to stderr */
+   // int temponly; /* if non zero, print only temperature to stderr */ update NC
    int dimer; /* if non zero, dimer structure is calculated */
 } thal_args;
 
 
 /* Structure for receiving results from the thermo alignment calculation */
-typedef struct {
+typedef struct thal_results {
    char msg[255];
    int no_structure;    // Added no structure (1 if no structure found)
    double temp;
@@ -118,8 +118,46 @@ typedef struct {
    double dg;           // Added gibbs free energy value
    int align_end_1;
    int align_end_2;
+   char *sec_struct;
 } thal_results;
 
+
+/* The files from the directory primer3_config loaded as strings */
+typedef struct thal_parameters {
+  char *dangle_dh;
+  char *dangle_ds;
+  char *loops_dh;
+  char *loops_ds;
+  char *stack_dh;
+  char *stack_ds;
+  char *stackmm_dh;
+  char *stackmm_ds;
+  char *tetraloop_dh;
+  char *tetraloop_ds;
+  char *triloop_dh;
+  char *triloop_ds;
+  char *tstack_tm_inf_ds;
+  char *tstack_dh;
+  char *tstack2_dh;
+  char *tstack2_ds;
+} thal_parameters;
+
+/*
+ * THL_FAST    = 0 - score only with optimized functions (fast)
+ * THL_GENERAL = 1 - use general function without debug (slow)
+ * THL_DEBUG_F = 2 - debug mode with fast, print alignments on STDERR
+ * THL_DEBUG   = 3 - debug mode print alignments on STDERR
+ * THL_STRUCT  = 4 - calculate secondary structures as string
+ */
+typedef enum thal_mode {
+  THL_FAST    = 0,
+  THL_GENERAL = 1,
+  THL_DEBUG_F = 2,
+  THL_DEBUG   = 3,
+  THL_STRUCT  = 4
+} thal_mode;
+
+
 /*** END OF TYPEDEFS ***/
 
 void set_thal_default_args(thal_args *a);
@@ -137,7 +175,13 @@ void set_thal_oligo_default_args(thal_args *a);
     exit(-1);
   }
 #endif
-int  get_thermodynamic_values(const char* path, thal_results *o);
+int thal_set_null_parameters(thal_parameters *a);
+
+int thal_load_parameters(const char *path, thal_parameters *a, thal_results* o);
+
+int thal_free_parameters(thal_parameters *a);
+
+int get_thermodynamic_values(const thal_parameters *tp, thal_results *o);
 
 void destroy_thal_structures(void);
 
@@ -147,11 +191,20 @@ void destroy_thal_structures(void);
    to check errno.
 */
 
-void thal(const unsigned char *oligo1,
-	  const unsigned char *oligo2,
-	  const thal_args* a,
-	  thal_results* o,
-    const int print_output,
-    char *ascii_structure);
-
-#endif
+/* `print_output` is primer3-py modification argument to simplify conditionally
+* computing structure in calls to `thal` and directing output to `stdout` or to
+* the buffer in the `thal_results` `o->sec_struct` field if it is pre-allocated
+* to not be `NULL`. modified versions of `drawDimer` `drawHairpin` exist to do
+* this work if `print_output == 1` otherwise if `print_output == 0` new
+* functions `calcDimer` and `calcHairpin` are called to compute the `o->temp`
+* `o->ds`, `o->dh`, and `o->dg` fields.  Calls to `thal()` in `libprimer3.c`
+* always use `print_output == 1` for the design path
+*/
+void thal(
+      const unsigned char *oligo1,
+	   const unsigned char *oligo2,
+	   const thal_args* a,
+      const thal_mode mode,
+	   thal_results* o,
+      const int print_output); /* primer3-py modification argument NC */
+#endif /* _THAL_H */
